@@ -12,31 +12,31 @@ import matplotlib.pylab as pylab
 class Portfolio:
     """Class representing a portfolio of ETFs and his performances during time"""
 
-    INFO = 'Info.csv'
-
-    def __init__(self):
+    def __init__(self, info_file='Info.csv'):
         """
         Class initialization. The paramter neeeded is refering to the info file.
         :param info_file: str
         :return: None 
         """    
-        params = {'axes.titlesize': 'xx-large'}
+        params = {'axes.titlesize': 'xx-large', 'axes.labelsize': 6, 'font.size':6}
         pylab.rcParams.update(params)
         self.etfs = {}
-        if os.path.isfile(self.INFO):
-            f = pd.read_csv(self.INFO, parse_dates=True)
+        self.infoFile = info_file
+        if os.path.isfile(self.infoFile):
+            f = pd.read_csv(self.infoFile, parse_dates=True)
+            basePath = self.infoFile.split('Info.csv')[0]
             for x in f.index:
                 if not isinstance(f.loc[x, 'sell_date'], str):
                     self.etfs[f.loc[x, 'Name']] = ETF(f.loc[x, 'Name'],datetime.strptime(f.loc[x, 'buy_date'],'%Y-%m-%d').date(),
-                    f.loc[x, 'n_shares'],f.loc[x, 'buy_price'], f.loc[x, 'commissions_ini'])
+                    f.loc[x, 'n_shares'],f.loc[x, 'buy_price'], f.loc[x, 'commissions_ini'], info=f'{basePath}ETFs/')
                 else:
                     self.etfs[f.loc[x, 'Name']] = ETF(f.loc[x, 'Name'],datetime.strptime(f.loc[x, 'buy_date'],'%Y-%m-%d').date(),
                     f.loc[x, 'n_shares'],f.loc[x, 'buy_price'],f.loc[x, 'commissions_ini'],datetime.strptime(str(f.loc[x, 'sell_date']),'%Y-%m-%d').date(),
-                    f.loc[x, 'sell_price'],f.loc[x, 'info'],f.loc[x, 'sell_commissions'])
+                    f.loc[x, 'sell_price'], f'{basePath}ETFs/', f.loc[x, 'sell_commissions'])
         else:
             os.makedirs('ETFs')
             info = pd.DataFrame(columns=['Name','buy_date','n_shares','buy_price','commissions_ini','sell_date','sell_price','info','sell_commissions'])
-            info.to_csv(self.INFO, index=False)
+            info.to_csv(self.infoFile, index=False)
         self.data = pd.DataFrame()
         if len(self.etfs) != 0:
             self.refresh()
@@ -49,9 +49,9 @@ class Portfolio:
         """ 
         assert isinstance(etf, ETF), 'Error! You have to pass an ETF object as an argument'
         self.etfs[etf.ticker_name] = etf
-        f = pd.read_csv(self.INFO, parse_dates=True)
+        f = pd.read_csv(self.infoFile, parse_dates=True)
         f.loc[len(f)] = [etf.ticker_name,etf.buy_date,etf.n_shares, etf.buy_price, etf.commissions[0],etf.sell_date, etf.sell_price, etf.info,  etf.commissions[1]]
-        f.to_csv(self.INFO, index=False)
+        f.to_csv(self.infoFile, index=False)
         self.refresh()
     
     def sell_etf(self, etf_name, sell_date, sell_price, commissions):
@@ -68,11 +68,11 @@ class Portfolio:
         assert isinstance(sell_price, float), 'Sell_price must be float'
         assert isinstance(commissions, float), 'Commissions must be float'
         self.etfs[etf_name].sell(sell_date, sell_price, commissions)
-        new_file = pd.read_csv(self.INFO, index_col='Name')
+        new_file = pd.read_csv(self.infoFile, index_col='Name')
         new_file.loc[etf_name, 'sell_date'] = sell_date
         new_file.loc[etf_name, 'sell_price'] = sell_price
         new_file.loc[etf_name, 'sell_commissions'] = commissions
-        new_file.to_csv(self.INFO)
+        new_file.to_csv(self.infoFile)
         self.refresh()
 
     def remove_etf(self, etf_name):
@@ -83,46 +83,100 @@ class Portfolio:
         """ 
         assert etf_name in self.etfs.keys(), 'ETF not in portfolio'
         self.etfs.pop(etf_name)
-        new_file = pd.read_csv(self.INFO, index_col='Name').drop(etf_name,axis=0)
-        new_file.to_csv(self.INFO)
+        new_file = pd.read_csv(self.infoFile, index_col='Name').drop(etf_name,axis=0)
+        new_file.to_csv(self.infoFile)
         os.remove(f'ETFs/{etf_name}.csv')
         self.refresh()
         
-    def invested_amount(self):
+    def invested_amount(self, day='today'):
         """
         Calculates amount that was invested by summing every etf's initial investment.
+        :param day: datetime.date
         :return: float 
         """
-        return self.data['Invested'][-1]      
+        assert day == 'today' or isinstance(day, date), 'Error! You have to pass a datetime.date istance to the day parameter.'
+        if day == 'today':
+            day = self.data.index[-1]
+        if self.data.index[-1] >= day >= self.data.index[0]:
+            day = self._first_good_date(day)
+            return round(self.data.loc[day, 'Invested'], 2)
+        else:
+            return 0
     
-    def profit_loss(self, pct=False):
+    def profit_loss(self, day='today', pct=False):
         """
         Calculates the profit or loss accumulated since the beginning of the investments. 
-        If pct is True the result will be in percentage
+        If pct is True the result will be in percentage.
+        :param day: datetime.date
         :param pct: float
         :return: float 
         """
         assert isinstance(pct, bool), 'Error! The pct parameter must be boolean.'
-        return round(self.data['Profit/Loss%'][-1], 2) if pct else round(self.data['Profit/Loss'][-1], 2)
+        assert day == 'today' or isinstance(day, date), 'Error! You have to pass a datetime.date istance to the day parameter.'
+        if day == 'today':
+            day = self.data.index[-1]
+        if self.data.index[-1] >= day >= self.data.index[0]:
+            day = self._first_good_date(day)
+            if pct:
+                return round(self.data.loc[day, 'Profit/Loss%'], 2)
+            else:
+                return round(self.data.loc[day, 'Profit/Loss'], 2)
+        else:
+            return 0
+    
+    def annualized_gains(self, day='today'):
+        """
+        Calculates the annualized returns of the portfolio from the beginning to the date passed as a parameter.
+        :param day: datetime.date
+        :return float
+        """
+        assert day == 'today' or isinstance(day, date), 'Error! You have to pass a datetime.date istance to the day parameter.'
+        if day == 'today':
+            day = self.data.index[-1]
+        if self.data.index[-1] >= day >= self.data.index[0]:
+            day = self._first_good_date(day)
+            initialValue = self.invested_amount(day)
+            finalValue = self.value(day)
+            numberOfDays = (day - self.data.index[0]).days
+            return round(((finalValue / initialValue)**(365/numberOfDays) - 1) * 100, 2) 
+        else:
+            return 0
     
     def get_etfs_list(self):
         """
         Print the list of ETFs present in your portfolio
         :return: str 
         """
-        return " - ".join(self.etfs.keys())
+        return list(self.etfs.keys())
 
     def get_etf_by_name(self, ticker):
         """
         Returns the ETF object corrsponding to the given ticker. Remember that at the end of the ticket name a "-"
         and a number ar added. Check the etfs list to be sure about the names.
-        : param ticker: str
+        :param ticker: str
         :return: ETF
         """
         try:
             return self.etfs[ticker]
         except KeyError:
-            raise KeyError('Ticker not in Portfolio. Check "get_etf_list" to get the full list of ETFs.')
+            raise KeyError(f'<{ticker}> ticker not in Portfolio. Check "get_etf_list" to get the full list of ETFs.')
+    
+    def portfolioCountries(self):
+        """
+        Determines how many countries is the portfolio based.
+        :return: str
+        """
+        countries = []
+        for etfName in self.etfs.keys():
+            if '.' in etfName:
+                country = etfName.split('-')[0].split('.')[1]
+                countries.append(country)
+            else:
+                countries.append('NY')
+        if len(set(countries)) > 1:
+            return 'Mix'
+        else:
+            return list(set(countries))[0]
 
     def _first_good_date(self, day):
         """
@@ -148,12 +202,14 @@ class Portfolio:
         assert day == 'today' or isinstance(day, date), 'Error! You have to pass a datetime.date istance to the day parameter.'
         assert isinstance(pct, bool), 'Error! The pct parameter must be boolean.'
         if day == 'today':
-            day = self.data.index[-1].date()
-        assert self.data.index[-1].date() >= day >= self.data.index[0].date(), 'Invalid Date'
+            day = self.data.index[-1]
+        assert self.data.index[-1] >= day >= self.data.index[0], 'Invalid Date'
         day = self._first_good_date(day)
-        if day == self.data.index[0].date():
+        if day == self.data.index[0]:
+            day = day
             return round(self.data.loc[day, 'Profit/Loss%'], 2) if pct else round(self.data.loc[day, 'Profit/Loss'], 2) 
         else:
+            day = day
             return round(self.data.loc[day, 'Gains%'], 2) if pct else  round(self.data.loc[day, 'Gains'], 2)
     
     def value(self, day='today'):
@@ -164,8 +220,8 @@ class Portfolio:
         """
         assert day == 'today' or isinstance(day, date), 'Error! You have to pass a datetime.date istance to the day parameter.'
         if day == 'today':
-            day = self.data.index[-1].date()
-        if self.data.index[-1].date() >= day >= self.data.index[0].date():
+            day = self.data.index[-1]
+        if self.data.index[-1] >= day >= self.data.index[0]:
             day = self._first_good_date(day)
             return round(self.data.loc[day, 'Value'], 2)
         else:
@@ -183,13 +239,14 @@ class Portfolio:
         assert isinstance(date_ini, date), 'Error! You have to pass a datetime.date istance to date parameters.'
         assert isinstance(pct, bool), 'Error! The pct parameter must be boolean.'
         if date_fin == 'today':
-            date_fin = self.data.index[-1].date()
-        assert date_ini >= self.data.index[0].date(), 'Error ! Invalid Initial Date'
-        assert date_fin >= self.data.index[0].date(), 'Error ! Invalid Final Date'
+            date_fin = self.data.index[-1]
+        assert date_ini >= self.data.index[0], 'Error ! Invalid Initial Date'
+        assert date_fin >= self.data.index[0], 'Error ! Invalid Final Date'
         date_fin = self._first_good_date(date_fin)
-        if date_ini == self.data.index[0].date():
+        if date_ini == self.data.index[0]:
             profit = self.data.loc[date_fin, 'Profit/Loss']
         else:
+            #date_ini = self._first_good_date(self._first_good_date(date_ini) - timedelta(1))
             date_ini = self._first_good_date(date_ini - timedelta(1))
             profit = self.data.loc[date_fin, 'Profit/Loss'] - self.data.loc[date_ini, 'Profit/Loss']
         if pct:
@@ -197,7 +254,7 @@ class Portfolio:
         else:
             return round(profit, 2)
     
-    def equity_line(self, sp500=False, pct=True):
+    def equity_line(self, sp500=False, pct=False):
         """
         Draws an historical series about the Portfolio's gains. If the sp500 parameter is True, the graph of the
         Standard & Poor 500 is added. If the pct parameter is True results will be in percentage form.
@@ -213,18 +270,19 @@ class Portfolio:
             spy = yf.Ticker("SPY").history(start=self.data.index[0])
 
         # Plot the graph
-        fig = plt.figure(figsize=(10,4), dpi=300)
-        ax = fig.add_axes([0,0,1,1])
+        fig = plt.figure(figsize=(4, 2), dpi=200)
+        fig.patch.set_facecolor('#ececec')
+        ax = fig.add_subplot(111)
         ax.set_xlabel('Time')
         if pct:
-            ax.plot(self.data['Profit/Loss%'], lw=1.5, color="blue", label='Equity')
+            ax.plot(self.data['Profit/Loss%'], lw=1.2, color="blue", label='Equity')
             ax.set_title('Profit/Loss % - Daily')
             ax.set_ylabel('P/L %')
             if sp500:
                 spy['Var%'] = (spy.Close - spy.Close[0]) / spy.Close[0] * 100
-                ax.plot(spy['Var%'], color="orange", label="SPY", linewidth=1.5)
+                ax.plot(spy['Var%'], color="orange", label="SPY", linewidth=1.2)
         else:
-            ax.plot(self.data['Profit/Loss'], lw=1.5, label='Equity')
+            ax.plot(self.data['Profit/Loss'], lw=1.2, label='Equity')
             ax.set_ylabel('Gains (€)')
             ax.set_title('Profit/Loss (€) - Daily')
         ax.xaxis.set_major_locator(dates.MonthLocator())
@@ -232,7 +290,7 @@ class Portfolio:
         ax.grid(True)
         fig.autofmt_xdate()
         ax.legend()
-        return
+        return fig, ax
     
     def _get_marks(self, investment=True):
         """
@@ -247,7 +305,7 @@ class Portfolio:
         inv.dropna(inplace=True)
         marks = pd.DataFrame(index=inv.index)
         marks['Marks'] = np.where(inv['Invested'] == inv['Invested'].shift(1), round(inv['Invested'].shift(-1) - inv['Invested'],2), np.nan)
-        marks['Marks'][0] = inv['Invested'][0]
+        marks['Marks'][0] = round(inv['Invested'][0], 2)
         marks['Invested'] = inv['Invested']
         marks.dropna(inplace=True)
         if investment:
@@ -262,9 +320,12 @@ class Portfolio:
         :return: None
         """
         inv, marks = self._get_marks()
-        fig = plt.figure(figsize=(10,4), dpi=300)
-        ax = fig.add_axes([0,0,1,1])
-        ax.plot(inv['Invested'], lw=1.5, color="blue", label='Invested', marker="o", markersize=6, markerfacecolor="grey")
+        fig = plt.figure(figsize=(4, 2), dpi=200)
+        fig.patch.set_facecolor('#ececec')
+        ax = fig.add_subplot(111)
+        investmentValues = inv['Invested']
+        #investmentValues = pd.Series([0], index=[investmentValues.index[0]-timedelta(1)]).append(investmentValues)
+        ax.plot(investmentValues, lw=1.2, color="blue", label='Invested', marker="o", markersize=3, markerfacecolor="grey")
         ax.set_xlabel('Time')
         ax.set_ylabel('Investments (€)')
         ax.set_title('Investment Amount (€) - Daily')
@@ -273,14 +334,13 @@ class Portfolio:
         for x, y, mark in zip(marks.index, marks['Invested'], marks['Marks']):
             a = ax.get_ylim()
             if x == marks.index[0]:
-                ax.annotate(str(mark) + " €", xy=(x + timedelta(abs((self.data.index[0] - self.data.index[-1]).days) / 80), y + (a[1]-a[0])/35))
+                ax.annotate(str(mark) + " €", xy=(x + timedelta(abs((self.data.index[0] - self.data.index[-1]).days) / 80), y + (a[1]-a[0])/35), fontsize=5)
             else:
-                ax.annotate(str(mark) + " €", xy=(x + timedelta(abs((self.data.index[0] - self.data.index[-1]).days) / 50), y - (a[1]-a[0])/35))
+                ax.annotate(str(mark) + " €", xy=(x + timedelta(abs((self.data.index[0] - self.data.index[-1]).days) / 50), y - (a[1]-a[0])/35), fontsize=5)
         ax.grid(True)
         fig.autofmt_xdate()
         ax.legend()
-        plt.show()
-        return 
+        return fig, ax
     
     def value_line(self):
         """
@@ -290,11 +350,12 @@ class Portfolio:
         """
         marks = self._get_marks(False)
         marks['Val'] = self.data['Value']
-        fig = plt.figure(figsize=(10,4), dpi=300)
-        ax = fig.add_axes([0,0,1,1])
-        ax.plot(self.data['Value'], alpha=0.8, lw=2, color="green", label='Value')
-        ax.scatter([x.date() for x in marks[marks['Marks']>0].index], marks[marks['Marks']>0]['Val'], marker='^', s=60, c="b", label="Buy")
-        ax.scatter([x.date() for x in marks[marks['Marks']<0].index], marks[marks['Marks']<0]['Val'], marker='v', s=60, c="r", label="Sell")
+        fig = plt.figure(figsize=(4,2), dpi=200)
+        fig.patch.set_facecolor('#ececec')
+        ax = fig.add_subplot(111)
+        ax.plot(self.data['Value'], alpha=0.8, lw=1.2, color="green", label='Value')
+        ax.scatter([x for x in marks[marks['Marks']>0].index], marks[marks['Marks']>0]['Val'], marker='^', s=20, c="b", label="Buy")
+        ax.scatter([x for x in marks[marks['Marks']<0].index], marks[marks['Marks']<0]['Val'], marker='v', s=20, c="r", label="Sell")
         ax.set_xlabel('Time')
         ax.set_ylabel('Portfolio\'s Value (€)')
         ax.set_title('Portfolio\'s Value (€) - Daily')
@@ -303,18 +364,18 @@ class Portfolio:
         for x, y, mark in zip(marks.index, marks['Val'], marks['Marks']):
             a = ax.get_ylim()
             if x == marks.index[0]:
-                ax.annotate(str(mark) + " €", xy=(x + timedelta(abs((self.data.index[0] - self.data.index[-1]).days) / 80), y + (a[1]-a[0])/35))
+                ax.annotate(str(mark) + " €", xy=(x + timedelta(abs((self.data.index[0] - self.data.index[-1]).days) / 80), y + (a[1]-a[0])/35), fontsize=5)
             else:
                 if mark > 0:
-                    ax.annotate(str(mark) + " €", xy=(x + timedelta(abs((self.data.index[0] - self.data.index[-1]).days) / 60), y - (a[1]-a[0])/35))
+                    ax.annotate(str(mark) + " €", xy=(x + timedelta(abs((self.data.index[0] - self.data.index[-1]).days) / 60), y - (a[1]-a[0])/35), fontsize=5)
                 else:
-                    ax.annotate(str(mark) + " €", xy=(x - timedelta(abs((self.data.index[0] - self.data.index[-1]).days) / 15), y + (a[1]-a[0])/35))
+                    ax.annotate(str(mark) + " €", xy=(x - timedelta(abs((self.data.index[0] - self.data.index[-1]).days) / 15), y + (a[1]-a[0])/35), fontsize=5)
         ax.grid(True)
         fig.autofmt_xdate()
         ax.legend()
-        return 
+        return  fig, ax
     
-    def bar_chart(self, period, annot=True):
+    def bar_chart(self, period='M', annot=True):
         """
         Draws a bar chart of the Portfolio's gains grouped by the selected period. 
         You can choose between:
@@ -330,12 +391,15 @@ class Portfolio:
         assert isinstance(annot, bool), 'Error! Annot parameter must be boolean'
         months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
         periods = {"M": ("Monthly","Months"), "Y": ("Yearly", "Years"), "W": ("Weekly", "Weeks")}
-        sample = pd.concat([self.data.head(1), self.data.resample(period).last()])
+        data = self.data.copy()
+        data.set_index(pd.to_datetime(data.index), inplace=True)
+        sample = pd.concat([data.head(1), data.resample(period).last()])
         sample['Var%'] = (sample['Profit/Loss'] - sample['Profit/Loss'].shift(1)) / sample['Value'].shift(1) * 100 
         sample.dropna(inplace=True)
         colors = sample['Var%'].apply(lambda x: "green" if x > 0 else "red")
-        fig = plt.figure(figsize=(10,5), dpi=250)
-        ax = fig.add_axes([0,0,1,1])
+        fig = plt.figure(figsize=(4,2), dpi=200)
+        fig.patch.set_facecolor('#ececec')
+        ax = fig.add_subplot(111)
         ax.set_xlabel(periods[period][1])
         ax.set_ylabel("Var (%)")
         ax.set_title(f"{periods[period][0]} Profit / Loss %")
@@ -345,14 +409,16 @@ class Portfolio:
             labels = [x for x in sample.index.year]
             ax.set_ylim(sample['Var%'].min()-2,sample['Var%'].max()+2)  
         elif period == "W":
-            sample_M = pd.concat([self.data.head(1), self.data.resample("M").last()])
-            ax.set_xticks(np.arange(0, len(sample), 4))
-            labels = [m + "-" + y for m, y in zip([months[x-1] for x in sample_M.index.month], [str(x) for x in sample_M.index.year])]
-            labels[0] = months[months.index(labels[0][:-5]) - 1] + labels[0][-5:]
+            sample_M = pd.concat([data.head(1), data.resample("M").last()])
+            ax.set_xticks(np.arange(-2, len(sample_M)*4-2, 4))
+            labels = [m + "-" + y for m, y in zip([months[x-1] for x in sample_M.index.month[1:]], [str(x) for x in sample_M.index.year[1:]])]
+            m = months[int(months.index(labels[-1][:-5])) + 1] if int(months.index(labels[-1][:-5])) + 1 != 12 else months[0]
+            y = int(labels[-1][-4:]) if m != 0 else int(labels[-1][-4:]+1)
+            labels.append(m + '-' + str(y))
         else:
             labels = [m + "-" + y for m, y in zip([months[x-1] for x in sample.index.month], [str(x) for x in sample.index.year])]
         ax.set_xticklabels(labels)
-        cords = {'M': (0.2, 0.5, 'medium', 1), 'W': (0.5, 0.5, 'x-small', 1), 'Y': (0.045, 0.3, 'x-large', 0.85)}
+        cords = {'M': (0.2, 0.5, 4, 1), 'W': (0.5, 0.5, 'x-small', 1), 'Y': (0.045, 0.3, 'x-large', 0.85)}
         if annot:
             for d, v in zip(range(len(sample)), sample['Var%']):
                 if v > 0:
@@ -363,7 +429,7 @@ class Portfolio:
             fig.autofmt_xdate()
         ax.grid(True, alpha=0.5)
         ax.legend()
-        return
+        return fig, ax
 
     def pie_chart(self, pct=True, day='today'):
         """
@@ -379,16 +445,16 @@ class Portfolio:
             return str(format(round(pct/100.*np.sum(allvals), 2),".2f")) + "€"
         vals = {}
         for name, etf in self.etfs.items():
-            name = name.split('-')[0].split('.')[0]
             if etf.get_value(day) != 0:
-                if name in vals:
-                    vals[name] += etf.get_value(day)
+                if name.split('-')[0].split('.')[0] in vals:
+                    vals[name.split('-')[0].split('.')[0]] += etf.get_value(day)
                 else:
-                    vals[name] = etf.get_value(day)
+                    vals[name.split('-')[0].split('.')[0]] = etf.get_value(day)
         wgt_values = [round(v, 2) for v in vals.values()]
         explode = [0 if x != max(wgt_values) else 0.06 for x in wgt_values]
-        fig = plt.figure(figsize=(4,2), dpi=300)
-        ax = fig.add_axes([0,0,1,1])
+        fig = plt.figure(figsize=(4,2), dpi=200)
+        ax = fig.add_subplot(111)
+        fig.patch.set_facecolor('#ececec')
         if pct:
             wedges, texts, autotexts = ax.pie(wgt_values, explode=explode, labels=vals.keys(), autopct='%1.1f%%')
             plt.setp(autotexts, size=5)
@@ -398,7 +464,7 @@ class Portfolio:
             plt.setp(autotexts, size=4)
         plt.setp(texts, size=7, family='monospace')
         ax.set_title('Portfolio\'s Composition', size='large', color='red', weight='bold')
-        return
+        return fig, ax
     
     # Just a try
     def scatter_plot(self):
@@ -423,7 +489,7 @@ class Portfolio:
         table['P/L (%)'] = [etf.profit_loss(pct=True) for etf in self.etfs.values()]
         return table
 
-    def porfolio_table(self):
+    def portfolio_table(self):
         """
         Creates a table showing statistics for each ETF type
         :return pandas.DataFrame
@@ -431,18 +497,36 @@ class Portfolio:
         idx = set(name.split('-')[0].split('.')[0] for name, etf in self.etfs.items() if not etf.sold())
         table = pd.DataFrame({'Invested': 0, 'Shares':0, 'Share Price':0, 'Present Value':0, 'P/L':0, 'P/L%':0},index=idx)
         for name, etf in self.etfs.items():
-            name = name.split('-')[0].split('.')[0]
             if not etf.sold():
-                table.loc[name, 'Invested'] += etf.initial_investment()
-                table.loc[name, 'Shares'] += etf.n_shares
-                table.loc[name, 'Share Price'] = etf.stock_price()
-                table.loc[name, 'Present Value'] += etf.present_value()
-                table.loc[name, 'P/L'] += etf.profit_loss()
+                table.loc[name.split('-')[0].split('.')[0], 'Invested'] += etf.initial_investment()
+                table.loc[name.split('-')[0].split('.')[0], 'Shares'] += etf.n_shares
+                table.loc[name.split('-')[0].split('.')[0], 'Share Price'] = etf.stock_price()
+                table.loc[name.split('-')[0].split('.')[0], 'Present Value'] += etf.present_value()
+                table.loc[name.split('-')[0].split('.')[0], 'P/L'] += etf.profit_loss()
         table.insert(1, 'PMA', round(table['Invested'] / table['Shares'], 2))
         table.insert(3, 'Initial Weight', round(table['Invested'] / table['Invested'].sum() * 100, 2))
         table.insert(4, 'Present Weight', round(table['Present Value'] / table['Present Value'].sum() * 100, 2))
         table['P/L%'] = round(table['P/L'] / table['Invested'] * 100, 2)
+        table['P/L'] = round(table['P/L'], 2)
+        table['Present Value'] = round(table['Present Value'], 2)
         return table.sort_values('Invested', 0, ascending=False)
+    
+    def last_day_table(self):
+        """
+        Creates a table showing statistics for each ETF type about the last trading day
+        :return pandas.DataFrame
+        """
+        idx = set(name.split('-')[0].split('.')[0] for name, etf in self.etfs.items() if not etf.sold()).union(['Totale'])
+        table = pd.DataFrame({'Value':0, 'P/L':0, 'P/L%':0},index=idx)
+        if len(self.etfs) > 0:
+            for name, etf in self.etfs.items():
+                if not etf.sold():
+                    table.loc[name.split('-')[0].split('.')[0], 'P/L'] += etf.get_gain()
+                    table.loc[name.split('-')[0].split('.')[0], 'Value'] += etf.get_value(self.data.index[-1]-timedelta(1))
+            table['P/L%'] = round(table['P/L'] / table['Value'] * 100, 2)
+            table.loc['Totale', 'P/L'] = table['P/L'].sum()
+            table.loc['Totale', 'P/L%'] = self.gains(pct=True)
+            return table.sort_values('Value', 0, ascending=False)
 
     def refresh(self):
         """
@@ -450,6 +534,13 @@ class Portfolio:
         Gains - Gains%
         :return: None
         """
+        lastDate = max(etf.data.index[-1] for etf in self.etfs.values())
+        for etf in self.etfs.values():
+            isLastDayMissing = etf.data.index[-1] < lastDate
+            if isLastDayMissing and not etf.sold():
+                lastDay = pd.DataFrame([etf.data.iloc[-1]], columns=etf.data.columns, index=[lastDate])
+                etf.data = etf.data.append(lastDay)
+                etf.calculateStats()
         # Get Profit/Loss series
         p_l = pd.DataFrame()
         for name, etf in self.etfs.items():
@@ -467,6 +558,6 @@ class Portfolio:
         self.data['Invested'] = inv.sum(axis=1)
 
         self.data['Profit/Loss%'] = self.data['Profit/Loss'] / self.data['Invested'] * 100 # Calculates the Profit/Loss (%)
-        self.data['Value'] = self.data['Invested'] + self.data['Profit/Loss']
+        self.data['Value'] = round(self.data['Invested'] + self.data['Profit/Loss'], 2)
         self.data['Gains'] = self.data['Profit/Loss'] - self.data['Profit/Loss'].shift(1)
         self.data['Gains%'] = self.data['Gains'] / self.data['Value'].shift(1) * 100
